//******************************************************************************
// OpenGL Radiosity example unit by Georgy Moshkin
// -----------------------------------------------
// email  : tmtlib@narod.ru
// WWW    : http://www.tmtlib.narod.ru/
//
//
// Description:
// ------------
// Radiosity lightmaps generation with OpenGL. Non physical correct.
// Calculation speed depends on FPS of your OpenGL video card.
//
// License:
// --------
// This unit is freely distributable without licensing fees and is
// provided without guarantee or warrantee expressed or implied. This unit
// is Public Domain. Feel free to use or enhance this code. You can use the
// program for any purpose you see fit.
//
//******************************************************************************
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  OpenGL,
  glBMP,   // TEXTURES UNIT
  _dream,  // RADIOSITY UNIT
  _vector; // VECTOR MATH UNIT

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Timer1: TTimer;
    RadioGroup1: TRadioGroup;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;

    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerDraw(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure lampclick(Sender: TObject);
  private
    { Private declarations }
    rc : HGLRC;    // RC
    dc  : HDC;     // DC

    procedure InitGL;
    procedure glDraw;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////


procedure TForm1.glDraw();
var i,j:integer;
begin

 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();

  glPolygonmode(GL_FRONT_AND_BACK, GL_FILL);

  gluLookAt(0,0,-100, //  
            0,0,0,    //  
            0,1,0);    //    

  glColor3f(1,1,1);

  glRotatef(GetTickCount/100,0,1,0);

  renderquad;

  SwapBuffers(DC);

end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
procedure TForm1.InitGL;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); 	   // White Background
  glShadeModel(GL_SMOOTH);                 // Enables Smooth Color Shading
  glClearDepth(1.0);                       // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);                 // Enable Depth Buffer
  glDepthFunc(GL_LESS);		           // The Type Of Depth Test To Do

  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   //Realy Nice perspective calculations

end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormCreate(Sender: TObject);
var pfd : TPIXELFORMATDESCRIPTOR;
    pf  : Integer;
begin
  DecimalSeparator:='.';
  dc:=GetDC(Panel3.Handle);

  pfd.nSize:=sizeof(pfd);
  pfd.nVersion:=1;
  pfd.dwFlags:=PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
  pfd.iPixelType:=PFD_TYPE_RGBA;
  pfd.cColorBits:=32;

  pf :=ChoosePixelFormat(dc, @pfd);
  SetPixelFormat(dc, pf, @pfd);

  rc :=wglCreateContext(dc);    // RC
  wglMakeCurrent(dc,rc);        // DC

  InitGL;

  updown:=1;
end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormResize(Sender: TObject);
begin
  glViewport(0, 0, Panel3.Width, Panel3.Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, Panel3.Width/Panel3.Height, 0.1, 500.0);

  glMatrixMode(GL_MODELVIEW);
  glDraw();
end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormDestroy(Sender: TObject);
begin
  wglMakeCurrent(0,0);
  wglDeleteContext(rc);
end;

procedure TForm1.TimerDraw(Sender: TObject);
begin
glDraw();
end;

procedure TForm1.Button1Click(Sender: TObject);
const ts=16; //    
var i,j,k:integer; //  
var mytex:string; //  ;
begin


    makeroom;

    //    CULLING:

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);


   // -= RADIOSITY BY GEORGY MOSHKIN =-

   radiositymode(32,32); //   Radosity
   CreateRadiosityMem;   //    "" Radiosity
   calculateFormFactor;  //   FormFactor  

   //  RADIOSITY:

   for i:=1 to 8 do
    begin
     RadiosityStep; //    RADIOSITY
    end;


   //   RADIOSITY:

   RadiosityEnd(Panel3.Width, Panel3.Height);

end;

procedure TForm1.Button2Click(Sender: TObject);
var i:integer;
begin

// SAVE LIGHTMAPS to light1.bmp, light2.bmp, light3.bmp, ...
for i:=0 to leveldim-1 do
 begin
   lmap[i].SaveImage('light'+inttostr(i)+'.bmp');
 end;


end;


procedure TForm1.Button3Click(Sender: TObject);
var i:integer;
begin


// MAKE 3d ROOM
makeroom;

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);


// LOAD LIGHTMAPS from bmp files
for i:=0 to leveldim-1 do
 begin
   lmap[i].LoadImage('light'+inttostr(i)+'.bmp');
   lmap[i].SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
   lmap[i].GenTexture(False, False);
 end;

end;

// This code is public domain, feel free to do with it any descent thing.
procedure TForm1.lampclick(Sender: TObject);
begin
if radiogroup1.ItemIndex=0 then updown:=1
 else updown:=-1;
end;

end.

