//******************************************************************************
// OpenGL Radiosity unit by Georgy Moshkin
// ---------------------------------------
// email  : tmtlib@narod.ru
// WWW    : http://www.tmtlib.narod.ru/
//
// Description:
// ------------
// Radiosity lightmaps generation with OpenGL. Non physical correct.
// Calculation speed depends on FPS of your OpenGL video card.
//
// License:
// --------
// This unit is freely distributable without licensing fees and is
// provided without guarantee or warrantee expressed or implied. This unit
// is Public Domain. Feel free to use or enhance this code. You can use the
// program for any purpose you see fit.
//
//******************************************************************************
unit _dream; // _DREAM.PAS RADIOSITY UNIT

interface

uses opengl,  //  OpenGL
     _vector, //   
     windows, // WinAPI
     glbmp;   //   

//        
// Ψ  tmtlib.narod.ru

type TRGB = record         //      RGB
             r,g,b:single; // (r - , g - , b - )
            end;

// ---------------------------------------------
// HELP     _dream.pas:
//      e-mail tmtlib@narod.ru
// Georgy Moshkin (http://www.tmtlib.narod.ru)

// RADIOSITYMODE PROCEDURE - INITIALISATION
procedure RadiosityMode(xw,xh:integer);
//   Radiosity ( viewport  XW  XH)
//   xw=xh. : 8x8, 16x16, 32x32, 64x64  ..
// /Example: RadiosityMode(32,32);
// :  XW  XH      ViewPort-,
//       ,  ""  .

// RADIOSITYEND PROCEDURE - VIEWPORT RESTORE
procedure RadiosityEnd(width,height:integer);
//     ViewPort  
// width, height. ,     OpenGL : 640x480
// /Example: RadiosityEnd(640,480);

// CREATERADIOSITYMEM PROCEDURE - CREATE STORAGE FOR VIEWPORT PIXELS
procedure CreateRadiosityMem;
//  ,     ,  "" 
//  ViewPort-  Radiosity.     XW  XH,
//    RadiosityMode(32,32);
// (  GetMem(RadiosityPointer, RadW*RadH*3); )
// /Example: CreateRadiosityTexture;
// :      
// Radiosity . RADIOSITY LIGHTMAPS GENERATION SPEED
// DEPENDS ON xw,xh

// CALCULATEFORMFACTOR PROCEDURE - CALCULATES FORM-FACTOR
procedure CalculateFormFactor;
//   Form-Factor,    
//   Radiosity.   Form-Factor  
//  ViewPort-  RadiosityTexture.
// : CalculateFormFactor;
// :     Form-Factor-, 
//    -.

// RADIOSITYSTEP PROCEDURE - DO ONE RADIOSITY-STEP
procedure RadiosityStep;
//       Radiosity,
//        
// : for i:=1 to ... do RadiosityStep;
// :     .

var updown:integer;
// RENDERQUAD PROCEDURE - RENDER SCENE
procedure RenderQuad;
//       level
//      Radiosity,
//       
// : RenderQuad;

procedure MakeRoom;


var RadW, RadH:integer; // ,    
                        // Radiosity- ( RadiosityPointer, ViewPort-
                        //  ).


type TQuad=record               //   
           p1,p2,p3,p4:TVector; //  
           normal:TVector;      // 
           tex:GLUINT;          // OpenGL-  
           end;


const leveldim=16;

var        lmap:array[0..leveldim-1] of TGLBMP;
           tmap:array[0..leveldim-1] of TGLBMP;

var level:array[0..leveldim-1] of TQUAD;     // 11 

var RadiosityPointer: Pointer; //      Radiosity

var formFactor:array[0..512-1, 0..512-1] of single; // -
                                                    //  
                                                    // RadW x RadH 



implementation

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


procedure setlmap(i:integer;mytex:string);
begin
lmap[i].Create(mytex);
lmap[i].SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
lmap[i].GenTexture(False, False);
end;

procedure settmap(i:integer;mytex:string);
begin
tmap[i].Create(mytex);
tmap[i].SetTextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
tmap[i].GenTexture(False, False);
end;


procedure MakeRoom;
var big:array[1..8] of TVector;
var average:array[1..8] of TVector;
var small:array[1..8] of TVector;
        i:integer;


begin

big[1]:=makethis(-20, -20, 20);
big[2]:=makethis(20, -20 ,20);
big[3]:=makethis(20, -20,-20);
big[4]:=makethis(-20, -20, -20);

big[5]:=makethis(-20, 20, 20);
big[6]:=makethis(20,  20 ,20);
big[7]:=makethis(20,  20,-20);
big[8]:=makethis(-20, 20, -20);

average[1]:=makethis(-8.75, -20 ,1.75);
average[2]:=makethis(1,     -20, -3);
average[3]:=makethis(-4.25, -20, -14.25);
average[4]:=makethis(-14.25,-20,-9.75);

average[5]:=makethis(-8.75, 2.75 ,1.75);
average[6]:=makethis(1,     2.75, -3);
average[7]:=makethis(-4.25, 2.75, -14.25);
average[8]:=makethis(-14.25,2.75,-9.75);

small[1]:=makethis(2,    -20, 8.75);
small[2]:=makethis(13,   -20, 11.25);
small[3]:=makethis(15.5, -20, 1);
small[4]:=makethis(5,    -20, -1.5);

small[5]:=makethis(2,    -7, 8.75);
small[6]:=makethis(13,   -7, 11.25);
small[7]:=makethis(15.5, -7, 1);
small[8]:=makethis(5,    -7, -1.5);

for i:=0 to leveldim-1 do setlmap(i,'black.bmp');
for i:=0 to leveldim-1 do settmap(i,'white.bmp');

settmap(3,'red.bmp');
settmap(2,'green.bmp');

//setlmap(3,'white.bmp');
//setlmap(2,'white.bmp');


    glEnable(GL_TEXTURE_2D);
    glEnable(GL_CULL_FACE);


    (*
      level[0].map.SetPixel(0,0,0,255,0);
      level[0].map.GenTexture(False, False);
      *)



with level[0] do
 begin
  p1:=big[1];
  p2:=big[2];
  p3:=big[3];
  p4:=big[4];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[1] do
 begin
  p1:=big[5];
  p2:=big[8];
  p3:=big[7];
  p4:=big[6];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[2] do
 begin
  p1:=big[1];
  p2:=big[4];
  p3:=big[8];
  p4:=big[5];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[3] do
 begin
  p1:=big[3];
  p2:=big[2];
  p3:=big[6];
  p4:=big[7];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[4] do
 begin
  p1:=big[1];
  p2:=big[5];
  p3:=big[6];
  p4:=big[2];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[5] do
 begin
  p1:=big[4];
  p2:=big[3];
  p3:=big[7];
  p4:=big[8];
  normal:=NormalVector(p1,p2,p3);
 end;

// average

with level[6] do
 begin
  p1:=average[5];
  p2:=average[6];
  p3:=average[7];
  p4:=average[8];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[7] do
 begin
  p1:=average[1];
  p2:=average[5];
  p3:=average[8];
  p4:=average[4];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[8] do
 begin
  p1:=average[3];
  p2:=average[7];
  p3:=average[6];
  p4:=average[2];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[9] do
 begin
  p1:=average[1];
  p2:=average[2];
  p3:=average[6];
  p4:=average[5];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[10] do
 begin
  p1:=average[4];
  p2:=average[8];
  p3:=average[7];
  p4:=average[3];
  normal:=NormalVector(p1,p2,p3);
 end;

// small

with level[11] do
 begin
  p1:=small[5];
  p2:=small[6];
  p3:=small[7];
  p4:=small[8];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[12] do
 begin
  p1:=small[1];
  p2:=small[5];
  p3:=small[8];
  p4:=small[4];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[13] do
 begin
  p1:=small[3];
  p2:=small[7];
  p3:=small[6];
  p4:=small[2];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[14] do
 begin
  p1:=small[1];
  p2:=small[2];
  p3:=small[6];
  p4:=small[5];
  normal:=NormalVector(p1,p2,p3);
 end;

with level[15] do
 begin
  p1:=small[4];
  p2:=small[8];
  p3:=small[7];
  p4:=small[3];
  normal:=NormalVector(p1,p2,p3);
 end;


end;


procedure RenderQuad; //  

var i:integer; //  

begin

  glDepthFunc(GL_LEQUAL);              // The Type Of Depth Testing (Less Or Equal)
  glEnable(GL_DEPTH_TEST);             // Enable Depth Testing




glDisable(GL_BLEND);

 for i:=0 to leveldim-1 do //    
 with level[i] do
  begin
   tmap[i].bind;          //   

   glBegin(GL_QUADS);

   glTexCoord2f(0,0);           //  
   glVertex3f(p1.x,p1.y,p1.z);  //  

   glTexCoord2f(1,0);
   glVertex3f(p2.x,p2.y,p2.z);

   glTexCoord2f(1,1);
   glVertex3f(p3.x,p3.y,p3.z);

   glTexCoord2f(0,1);
   glVertex3f(p4.x,p4.y,p4.z);

   glEnd;
  end;

    glBlendFunc(GL_ZERO, GL_SRC_COLOR);
    glEnable(GL_BLEND);
    glcolor4f(1,1,1,1);

 for i:=0 to leveldim-1 do //    
 with level[i] do
  begin
   lmap[i].bind;          //   

   glBegin(GL_QUADS);

   glTexCoord2f(0,0);           //  
   glVertex3f(p1.x,p1.y,p1.z);  //  

   glTexCoord2f(1,0);
   glVertex3f(p2.x,p2.y,p2.z);

   glTexCoord2f(1,1);
   glVertex3f(p3.x,p3.y,p3.z);

   glTexCoord2f(0,1);
   glVertex3f(p4.x,p4.y,p4.z);

   glEnd;
  end;

glDisable(GL_BLEND);
glDisable(GL_CULL_FACE);

// 

//glRotatef(90,1,0,0);
//glRotatef(-90,0,0,1);

  glDisable(GL_TEXTURE_2D);

  glBegin(GL_QUADS);
  glColor3f(1,1,1);

  glVertex3f(0-5,
             19*updown,
             0-5);

  glVertex3f(0+5,
             19*updown,
             0-5);

  glVertex3f(0+5,
             19*updown,
             0+5);

  glVertex3f(0-5,
             19*updown,
             0+5);

  glEnd;

  glEnable(GL_CULL_FACE);

  glEnable(GL_TEXTURE_2D);


end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure RadiosityMode(xw,xh:integer);  //  Radiosity
begin
  RadW:=xw; //   
  RadH:=xh;

  Randomize; //     

  glViewport(0, 0, RadW, RadH); //   viewport

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(90,RadW/RadH,0.1,1000);  //   90  !!!
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure RadiosityEnd(width,height:integer); //    Radiosity
begin
 glViewport(0, 0, width, height); //  ViewPort
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, width/height, 0.1, 1000.0); //   ,  = 45 !!!
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
end;

//     .   - 
function adj(inp:single):single;
var z:single;
begin
z:=inp;                   //     (R,G  B)
if (z>0) and (z<0.5) then // ,    
z:=sin(z*3.14/0.5);       //    
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure updateRadiosityTexture; //  ViewPort
var zzz:integer;
begin
  //    Z-BUFFER-
  glReadPixels(0, 0, RadW, RadH, GL_RGB, GL_UNSIGNED_BYTE, RadiosityPointer);
  // Ѩ! VIEWPORT   RadiosityPointer
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//   ,  
//     Form-Factor-
// (   RadiosityPointer)
function RadiosityColor:Trgb;

type TPixel = Record   // 
       R, G, B : Byte; //     SINGLE,   BYTE
     end;              // R,G,B - , ,   (0...255)

var I, J : Integer;    //  
    Pixel : ^TPixel;   // ,      
                       //   Tpixel- 

    R,G,B:single;      //    SINGLE (0...1)

begin

R:=0; // 
G:=0;
B:=0;

  for I :=0 to RadW-1 do    //     RadiosityPointer
    for J :=0 to RadH-1 do
    begin
      Pixel:=Pointer(Integer(RadiosityPointer) + 3*(i*RadW + j));
      R:=R+Pixel^.R * formFactor[i,j]; //  FormFactor
      G:=G+Pixel^.G * formFactor[i,j]; //    R,G,B
      B:=B+Pixel^.B * formFactor[i,j]; //   *
    end;

    result.R:=R/(RadW*RadH*255); //   Single
    result.G:=G/(RadW*RadH*255); //  0...255  0...1
    result.B:=B/(RadW*RadH*255); //     RadW*RadH

end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure CreateRadiosityMem; //    "" Radiosity
begin
  GetMem(RadiosityPointer, RadW*RadH*3); // 3 -    
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure calculateFormFactor; //   -
var i,j:integer;      //  
begin

//  FormFactor-     i,j
//    :
for i:=0 to RadW-1 do
 for j:=0 to RadH-1 do
    formFactor[i,j]:=0.5+0.5*sin( i*3.14/(RadW-1) )*sin( j*3.14/(RadH-1) );

end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//      "" Radiosity
//   Viewport.
function xrender(v_eye,v_dir,v_up:Tvector):TRGB; //   
begin
//    :
// v_eye -   (     )
// v_dir -   (  )
// v_up -   (,    )
//  :
//  ,    

    glClearColor(0.0,0.0,0.0,1); //    (0,0,0)
    glEnable(GL_DEPTH_TEST);     //  DEPTH TEST (Z-BUFFER)
    glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);  //    Z-BUFFER
    glLoadIdentity(); //  

    //      ( ):
                  gluLookAt(v_eye.x+v_dir.x/1000,v_eye.y+v_dir.y/1000,v_eye.z+v_dir.z/1000,
                            v_eye.x+v_dir.x,
                            v_eye.y+v_dir.y,
                            v_eye.z+v_dir.z,
                            v_up.x,v_up.y,v_up.z);

    //  :
    renderquad;

    //   VIEWPORT-    RADIOSITYPOINTER:
    updateRadiosityTexture;

    //      RADIOSITYPOINTER  ר FORMFACTOR:
    xrender:=RadiosityColor;

end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//  RADIOSITY (  )
procedure RadiosityStep;

var randvec:Tvector; //  
    perpvec:Tvector; //  

    v_eye,v_dir,v_up:Tvector; //  ,  
                              //   

    i,j,k,l:integer; //  

    basevector:TVector; //   ( )

    //     :
    uvector:TVector; //  U-
    vvector:TVector; //  V-

    u,v:integer; //      

    ulen,vlen:single; //    

    udx,vdx:single; //      uvector  vvector

    temprgb:TRGB; //     

begin


for i:=0 to leveldim-1 do //    
 with level[i] do
 begin
  basevector:=p1;               //   =   

  uvector:=vecminusvec(p2,p1);  //   
  vvector:=vecminusvec(p4,p1);

  ulen:=VectorLen(uvector); //   
  vlen:=VectorLen(vvector);

  udx:=ulen/(lmap[i].getWidth{-1}); //     
  vdx:=vlen/(lmap[i].getHeight{-1});

  uvector:=Normalize(uvector); //    ()
  vvector:=Normalize(vvector);

  //      :

  BaseVector:=VecPlusVec(basevector,

            VecPlusVec(Umnoj(uvector,udx/2),
                       Umnoj(vvector,vdx/2))
                       );

  // Ĩ       :

  for u:=0 to lmap[i].getWidth-1 do
   for v:=0 to lmap[i].getHeight-1 do
    begin

     randvec.x:=127-random(255); //  
     randvec.y:=127-random(255);
     randvec.z:=127-random(255);
     randvec:=Normalize(randvec); // 

     perpvec:=(CrossVector(normal,randvec)); //    ( )

     //   ,    
     v_eye:=VecPlusVec(basevector,

            VecPlusVec(Umnoj(uvector,udx*u),
                       Umnoj(vvector,vdx*v))
                       );

     //   =  
     v_dir:=normal;

     //  
     v_up:=perpvec;

     //        TEMPRGB
     TempRGB:=xrender(v_eye,v_dir,v_up);

     //    ,    
     lmap[i].AddPixel(u,v,round(255*TempRGB.R),
                      round(255*TempRGB.G),
                      round(255*TempRGB.B));


    end;



 end;


//  OPENGL- 
for i:=0 to leveldim-1 do
 with level[i] do
  lmap[i].GenTexture(False, False);
end;


// This code is public domain, feel free to do with it any descent thing.
// Author: Georgy V. Moshkin
end.
